


#include <dirent.h>
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <errno.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>

#include <dvdread/dvd_reader.h>
#include <dvdread/ifo_types.h>
#include <dvdread/ifo_read.h>


char* dvd_device="/dev/dvd";


#define KILOBYTE(n) ((n) * 1024)
#define MEGABYTE(n) ((n) * 1024 * 1024)

#define MAXPARSEBUFFER KILOBYTE(10)

#define MALLOC(type, size)  (type *)malloc(sizeof(type) * (size))




void analyze( char* device ) {

  dvd_reader_t *dvd;
  ifo_handle_t *vmg_file;
  tt_srpt_t *tt_srpt;

  // Open dvd
  dvd = DVDOpen( device );
  if( !dvd ) {
    // Error output for testing
    fprintf( stderr, "Cannot open DVD device\n" );
    // done
    return;
  }

  // Get Video Manager
  vmg_file = ifoOpen( dvd, 0 );
  if( !vmg_file ) {
    DVDClose( dvd );
    // Error output for testing
    fprintf( stderr, "Cannot read DVD information\n");
    // No DVD Titles found
    return;
  }

  // Get Title Information
  tt_srpt = vmg_file->tt_srpt;

  // Print number of chapters of every title
  //
  int i;
  for ( i = 0; i < tt_srpt->nr_of_srpts; i++ ) {

    // Print chapters in this title
    //
    printf( "Title %02d: %02d Chapters, ", i+1, tt_srpt->title[i].nr_of_ptts+1 );

    //
    // Load the VTS information for this title set.
    //
    ifo_handle_t *vts_file = ifoOpen( dvd, tt_srpt->title[i].title_set_nr );
    if ( vts_file != NULL ) {

      // Title
      char tmp1[50];
      strncpy( tmp1, vts_file->vtsi_mat->vts_identifier, 11 );
      tmp1[12]=0;
      printf( "id: %s, ", tmp1);
      if ( vts_file->vmgi_mat ) {
        strncpy( tmp1, vts_file->vmgi_mat->provider_identifier, 31 );
        tmp1[32]=0;
        printf( "provider: %s, ", tmp1);
      }
      else {
        printf( "provider: NONE, ");
      }

      if ( vts_file->txtdt_mgi ) {
        strncpy( tmp1, vts_file->txtdt_mgi->disc_name, 13 );
        tmp1[14]=0;
        printf( "disc name: %s, ", tmp1);
      }
      else {
        printf( "disc name: NONE, ");
      }




      // VTS Information retrieved
      if ( vts_file->vts_pgcit != NULL ) {

        printf("Languages: ");
        int j;
        for ( j=0; j<8; j++ ) {

          if ( vts_file->vts_pgcit->pgci_srp[0].pgc->audio_control[j] & 0x8000 ) {

            audio_attr_t * audio = &vts_file->vtsi_mat->vts_audio_attr[j];
            if ( audio->lang_type == 1 ) {

              int language = audio->lang_code;
              char tmp[3];
              tmp[0]=language>>8;
              tmp[1]=language&0xff;
              tmp[2]=0;
              printf("%s ", tmp);
            }
          }
        } // for
      } // if
    }
    ifoClose( vts_file );
    printf( "\n" );
  }

  // Close Video Manager and DVD and exit
  ifoClose( vmg_file );
  DVDClose( dvd );

  // Done
}



int main ( int argc , char ** argv ) {

  analyze( dvd_device );

  exit(0);

}
