/****************************************************************************
   This is free code written by Martin Fischer (martin.fischer@teamspace.de) 
 ****************************************************************************/


#include <dirent.h>
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <errno.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>

#include <dvdread/dvd_reader.h>
#include <dvdread/ifo_types.h>
#include <dvdread/ifo_read.h>


/****************************************************************************
 CHANGE THESE 4 LINES TO FIT YOUR SYSTEM!!
 ****************************************************************************/

char *dvd_device = "/dev/dvd";               // Your DVD Device
char *video_dir  = "/home/freigabe/Videos";  // Directory of dummy DVD file

char *preferred_language = "de";             // You prefere this language
char *accepted_languages = "de,en";          // But you would accept these too


/****************************************************************************
 NOTHING TO CONFIGURE BELOW
 ****************************************************************************/


#define KILOBYTE(n) ((n) * 1024)
#define MEGABYTE(n) ((n) * 1024 * 1024)

#define MAXPARSEBUFFER KILOBYTE(10)

#define MALLOC(type, size)  (type *)malloc(sizeof(type) * (size))



char *readline(FILE *f) {

  static char buffer[MAXPARSEBUFFER];
  if (fgets(buffer, sizeof(buffer), f) > 0) {
     int l = strlen(buffer) - 1;
     if (l >= 0 && buffer[l] == '\n')
        buffer[l] = 0;
     return buffer;
     }
  return NULL;
}



int getDVDTitles( char* device, int validTitles[] ) {

  // Return value: number of used elements
  // in the validTitles array:
  //
  int validTitleCount = 0;

  // Internal DVD Structures we use
  //
  dvd_reader_t *dvd;
  ifo_handle_t *vmg_file;
  tt_srpt_t *tt_srpt;

  // Open dvd device
  //
  dvd = DVDOpen( device );
  if( !dvd ) {
    // Error output for testing
    fprintf( stderr, "Cannot open DVD device\n" );
    // done
    return 0;
  }

  // Get Video Manager
  //
  vmg_file = ifoOpen( dvd, 0 );
  if( !vmg_file ) {
    DVDClose( dvd );
    // Error output for testing
    fprintf( stderr, "Cannot read DVD information\n");
    // No DVD Titles found
    return 0;
  }


  // Save Title Information, for later use
  //
  tt_srpt = vmg_file->tt_srpt;


  // Examine all Titles for audio information
  //
  int i;
  for ( i=0; i < tt_srpt->nr_of_srpts; i++ ) {

    // Load the VTS information for this title set.
    //
    ifo_handle_t *vts_file = ifoOpen( dvd, tt_srpt->title[i].title_set_nr );
    if ( vts_file != NULL ) {

      // VTS Information retrieved
      if ( vts_file->vts_pgcit != NULL ) {

        int j;
        for ( j=0; j<8; j++ ) {

          if ( vts_file->vts_pgcit->pgci_srp[0].pgc->audio_control[j] & 0x8000 ) {

            audio_attr_t * audio = &vts_file->vtsi_mat->vts_audio_attr[j];
            if ( audio->lang_type == 1 ) {

              int language = audio->lang_code;
              char tmp[3];
              tmp[0]=language>>8;
              tmp[1]=language&0xff;
              tmp[2]=0;

              // Is this the preferred language?
              //
              if ( strcmp( preferred_language, tmp ) == 0 ) {

                // Preferred language is found, overwrite any previous
                // existing entry, generated by an accepted language.

                // Check if we have already an entry of this title
                if ( ( validTitleCount > 0 ) && ( abs( validTitles[validTitleCount-1] ) == i+1 ) ) {
                  // Found the preferred language: use it
                  validTitles[validTitleCount-1] = i+1;
                }
                else {
                  // Found for the first time:
                  validTitles[validTitleCount] = i+1;
                  validTitleCount++;
                }
              }
              else if ( strstr( accepted_languages, tmp ) ) {

                // Accepted language found, only if not already an entry
                // for the preferred language is found

                // Check if we have already an entry of this title
                if ( ( validTitleCount > 0 ) && ( abs( validTitles[validTitleCount-1] ) == i+1 ) ) {
                  // Found a second time: nothing to do
                }
                else {
                  // Found for the first time:
                  validTitles[validTitleCount] = -(i+1);
                  validTitleCount++;
                }

              } // add title with accepted language
                 
            } // language available
       
          } // if

        } // for j
      } // if vts_pgcit

      // Close VTS Informations
      ifoClose( vts_file );

    } // if vts_file

  } // for i

  // Close Video Manager and DVD and exit
  ifoClose( vmg_file );
  DVDClose( dvd );

  // Done
  return validTitleCount;
}



int getExistingFiles( char* directory, int existingFiles[] ) {

  // The result of this function: number
  // of all existing files
  int fileCount = 0;

  // Prepare find command line
  //
  char *cmd=0;
  asprintf( &cmd, "find \"%s\" -follow -maxdepth 1 -type f -printf \"%%f\\n\" -name \"DVD-*\" 2>/dev/null ", directory );
  FILE *p = popen( cmd, "r" );
  if ( p ) {

    // Walk through all found files
    // and remember the title numbers
    //
    char *line;
    while ( line = readline(p) ) {

      // Check for a valid output line
      if ( *line && strlen( line ) >= 6 ) {
        // extract title number
        char *temp = (char *)strndup( line+4, 2 );
        // and put it in our array
        existingFiles[fileCount] = atoi( temp );
        fileCount++;
        // free used buffers
        free(temp);
      }
      free( line );

    } // while
    pclose(p);
  }

  free(cmd); 
  return fileCount;
}



void removeDVDTitleDummy( char* directory, int title ) {

  // create filename
  char *fullName=0;
  asprintf( &fullName, "%s/DVD-%02d", directory, title );
  unlink( fullName );
  asprintf( &fullName, "%s/DVD-%02d-Bonus", directory, title );
  unlink( fullName );
  free( fullName );
}



void createDVDTitleDummy( char* directory, int title ) {

  // create filename
  char *fullName=0;
  FILE *file;

  // Preferred or accepted title?
  //
  if ( title >= 0 ) {
    // Title in preferred language:
    asprintf( &fullName, "%s/DVD-%02d", directory, abs( title ) );
  }
  else {
    // Title in accepted language
    asprintf( &fullName, "%s/DVD-%02d-Bonus", directory, abs( title ) );
  }
    
  // Create file
  if ( file = fopen( fullName, "w" ) ) {
    fprintf( file, "DUMMY TITLE FILE - TITLE %02d\n", title );
    fclose( file );
  }

  free( fullName );
}




int main ( int argc , char ** argv ) {

  // Get number of existing DVD Title Files
  //
  int oldTitles[100];
  int oldTitleCount = getExistingFiles( video_dir, oldTitles );


  // Get number of current titles on DVD
  //
  int dvdTitles[100];
  int dvdTitleCount = getDVDTitles( dvd_device, dvdTitles );


  // Debugging output
  //
  fprintf( stderr, "Existing files %d\n", oldTitleCount );
  fprintf( stderr, "DVD Titles %d\n", dvdTitleCount );


  // Title Iterator
  //
  int title = 0;


  // Remove any old DVD Title File
  //
  for ( title=0; title < oldTitleCount; title++ ) {
    fprintf( stderr, "Removing DVD title: %02d\n", oldTitles[title] );
    removeDVDTitleDummy( video_dir, oldTitles[title] );
  }

  // Create DVD Title File for every valid title
  //
  int firstPreferredTitle = 0;
  for ( title=0; title < dvdTitleCount; title++ ) {
    // Create Dummy File for every found title
    fprintf( stderr, "Creating DVD title: %02d %s\n", abs( dvdTitles[title] ), ( dvdTitles[title]>0 ? "*" : "" ) );
    createDVDTitleDummy( video_dir, dvdTitles[title] );

    // Look for the first title in our preferred language
    if ( firstPreferredTitle == 0 && dvdTitles[title] > 0 ) {
      // Got it:
      firstPreferredTitle = dvdTitles[title];
    }
  }

  // Print preferred Title so it can be played
  printf( "%02d\n", firstPreferredTitle );
  
  // ready  
  //
  exit(0);

}
